<?php

namespace Drupal\scheduler_eck\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Creates scheduler plugins for all ECK entity types.
 */
class SchedulerEckDeriver extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    $instance = new static();
    $instance->entityTypeManager = $container->get('entity_type.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    if ($this->derivatives) {
      return $this->derivatives;
    }

    $storage = $this->entityTypeManager
      ->getStorage('eck_entity_type');

    $query = $storage->getQuery();
    $ids = $query
      ->condition($query->orConditionGroup()
        ->condition('published', 1)
        ->condition('published', TRUE))
      ->execute();

    foreach ($ids as $eck_type_id) {
      $derivative_key = sprintf('%s:%s', $base_plugin_definition['id'], $eck_type_id);
      $eck_type = $storage->load($eck_type_id);
      $description = $this->t('Provides support for scheduling @entityType entities', [
        '@entityType' => $eck_type->label(),
      ]);

      $this->derivatives[$eck_type_id] = $base_plugin_definition;
      $this->derivatives[$eck_type_id]['id'] = $derivative_key;
      $this->derivatives[$eck_type_id]['label'] = $eck_type->label();
      $this->derivatives[$eck_type_id]['description'] = $description;
      $this->derivatives[$eck_type_id]['entityType'] = $eck_type_id;
    }

    return $this->derivatives;
  }

}
